@@ -253,6 +253,7 @@ class CalculatedBondKeyFigureName(Enum):
     MacaulayDuration = "macdur"
     ModifiedDuration = "moddur"
     PriceClean = "price"
+    PVonTS = "pvonts"
     Spread = "spread"
     SpreadRisk = "spreadrisk"
     Yield = "yield"
@@ -282,5 +283,6 @@ class LiveBondKeyFigureName(Enum):
     LiborSpread6M = "libor 6m spread"
     Quote = "quote"
     Spread = "spread"
+    SpreadRisk = "spread risk"
     SwapSpread = "swap spread"
     Yield = "yield"
